/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { CodeEditor } from "@wso2is/react-components";
import { Divider, Tab } from "semantic-ui-react";
import { getInboundProtocolConfig, ApplicationInterface, SupportedAuthProtocolTypes } from "../../../../features/applications";
import { useDispatch } from "react-redux";
import { useTranslation } from "react-i18next";
import { store } from "../../../../features/core";
import { addAlert } from "@wso2is/core/dist/src/store";
import { AlertLevels } from "@wso2is/core/dist/src/models";

interface IntegrateAppComponentProps {
    applicationType: string;
    application: ApplicationInterface;
}

/**
 * Integrate Application flow component
 * @param props props required for component
 */
export const IntegrateAppComponent: FunctionComponent<IntegrateAppComponentProps> = (
    props: IntegrateAppComponentProps
): ReactElement => {

    const {
        applicationType,
        application
    } = props;

    const dispatch = useDispatch();
    const { t } = useTranslation();

    const deploymentConfigs = store.getState().config.deployment;
    const [ inboundProtocolConfigs, setInboundProtocolConfigs ] = useState(undefined);
    const [ authConfig, setAuthConfig ] = useState(undefined);
    const [ integrateCode, setIntegrateCode ] = useState("");

    // TODO : Move get configs to util function
    useEffect(() => {
        if ( application === undefined ) {
            return;
        }

        let protocolConfigs: any = {};
        const selectedProtocolList: string[] = [];

        application?.inboundProtocols?.map((protocol) => {

            const protocolName = mapProtocolTypeToName(protocol.type);

            getInboundProtocolConfig(application.id, protocolName)
                .then((response) => {
                    protocolConfigs = {
                        ...protocolConfigs,
                        [protocolName]: response
                    };

                    selectedProtocolList.push(protocolName);
                })
                .catch((error) => {
                    if (error?.response?.status === 404) {
                        return;
                    }

                    if (error?.response && error?.response?.data && error?.response?.data?.description) {
                        dispatch(addAlert({
                            description: error.response?.data?.description,
                            level: AlertLevels.ERROR,
                            message: t("devPortal:components.applications.notifications.getInboundProtocolConfig" +
                                ".error.message")
                        }));

                        return;
                    }

                    dispatch(addAlert({
                        description: t("devPortal:components.applications.notifications.getInboundProtocolConfig" +
                            ".genericError.description"),
                        level: AlertLevels.ERROR,
                        message: t("devPortal:components.applications.notifications.getInboundProtocolConfig" +
                            ".genericError.message")
                    }));
                })
                .finally(() => {
                    setInboundProtocolConfigs(protocolConfigs);
                });
        });
    },  [ application ]);

    useEffect(() => {
        if (inboundProtocolConfigs == undefined) {
            return;
        }

        const configs = {
            clientHost: deploymentConfigs.clientHost,
            clientID: inboundProtocolConfigs.oidc.clientId,
            loginCallbackURL: inboundProtocolConfigs.oidc.callbackURLs[0],
            logoutCallbackURL: inboundProtocolConfigs.oidc.callbackURLs[0],
            serverOrigin: deploymentConfigs.serverOrigin,
            tenant: deploymentConfigs.tenant,
            tenantPath: deploymentConfigs.tenantPath
        };

        setAuthConfig(configs);
        setIntegrateCode(`import { IdentityAuth } from "@wso2/identity-oidc-js";

/**
 * Minimal required configuration sample to initialize the client
 */
const authConfig = {
    loginCallbackURL: "` + configs?.loginCallbackURL + `",
    logoutCallbackURL: "` + configs?.logoutCallbackURL + `",
    clientHost: "` + configs?.clientHost + `",
    clientID: "` + configs?.clientID + `",
    serverOrigin: "` + configs?.serverOrigin + `"
};

/**
 * Initialize authClient
 */
const authClient = new IdentityAuth(authConfig);

/**
 * Trigger sign in
 */
authClient.signIn().then((response) => {
// Response with basic user details upon logged in
});
                            `);
    }, [ inboundProtocolConfigs ]);

    /**
     * Todo Remove this mapping and fix the backend.
     */
    const mapProtocolTypeToName = ((type: string): string => {
        let protocolName = type;
        if (protocolName === "oauth2") {
            protocolName = SupportedAuthProtocolTypes.OIDC;
        } else if (protocolName === "passivests") {
            protocolName = SupportedAuthProtocolTypes.WS_FEDERATION;
        } else if (protocolName === "wstrust") {
            protocolName = SupportedAuthProtocolTypes.WS_TRUST;
        } else if (protocolName === "samlsso") {
            protocolName = SupportedAuthProtocolTypes.SAML;
        }

        return protocolName;
    });

    // TODO : Sample code to be replaced with actual code in code editor
    const singlePageAppPanes = [
        { menuItem: 'React', render: () => (
            <Tab.Pane>
                <CodeEditor
                    showLineNumbers={ false }
                    language="javascript"
                    sourceCode={ integrateCode }
                    options={ {
                        gutters: []
                    } }
                    theme={ "light" }
                    readOnly={ true }
                />
            </Tab.Pane>
        ) },
        { menuItem: 'Angular', render: () => (
            <Tab.Pane>
                <CodeEditor
                    showLineNumbers={ false }
                    language="javascript"
                    sourceCode={ integrateCode }
                    options={ {
                        gutters: []
                    } }
                    theme={ "light" }
                    readOnly={ true }
                />
            </Tab.Pane>
        ) },
    ]

    const webAppPanes = [
        { menuItem: 'HTML', render: () => (
            <Tab.Pane>
                <CodeEditor
                    showLineNumbers={ false }
                    language="javascript"
                    sourceCode={ `<script 
    type="application/javascript" 
    src="http://unpkg.com/@wso2/identity-oidc-js@0.1.0/bundle/wso2-identity-oidc.standalone.js">
</script>

<script>

    var webAppOrigin = "` + authConfig?.loginCallbackURL + `";

    const authConfig = {
        // Web App URL
        clientHost: webAppOrigin,

        // ClientID generated for the application
        clientID: "` + authConfig?.clientID + `",

        // After login callback URL - We have use app root as this is a SPA 
        // (Add it in application OIDC settings "Callback Url")
        loginCallbackURL: webAppOrigin, 

        // After logout callback URL - We have use app root as this is a SPA 
        // (Add it in application OIDC settings "Callback Url")
        logoutCallbackURL: webAppOrigin,

        // WSO2 Identity Server URL
        serverOrigin: "https://asgardio.io",
            
        // Tenant Details: Enable if needed only
        // tenant: "demo.com",
        // tenantPath: "/t/demo.com"
    };

    </script>` }
                    options={ {
                        gutters: []
                    } }
                    theme={ "light" }
                    readOnly={ true }
                />
            </Tab.Pane>
        ) },
    ]

    return (
        <div className="quick-start-step-container">
            { applicationType !== "Web Application" &&
                <div className="quick-start-step">
                    <div className="step-content-container">
                        <h3>Install SDK</h3>
                        <div className="step-content">
                            <p>Run below command to install JavaScript OIDC SDK from NPM registry</p>
                            <code>npm install @wso2/identity-oidc-js</code>
                        </div>
                        <Divider hidden/>
                    </div>
                </div>
            }
            <div className="quick-start-step">
                <div className="step-content-container">
                    <h3>Add Asgardio to your app</h3>
                    <div className="step-content">
                        { applicationType === "Web Application" ?
                            <p>Copy and paste these scripts into the bottom of your <i>&lt;body&gt;</i> tag.</p>
                            :
                            <p>Copy and paste below code based on the technology that your app is built on.</p>
                        }
                        <Divider hidden/>
                        <Tab menu={ { secondary: true, pointing: true } } panes={ applicationType === "Web Application" ? webAppPanes : singlePageAppPanes } />
                    </div>
                    <Divider hidden/>
                </div>
            </div>
        </div>
    );
};
