/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { TestableComponentInterface } from "@wso2is/core/models";
import { Heading } from "@wso2is/react-components";
import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useSelector } from "react-redux";
import { Grid, Header, Card, Radio } from "semantic-ui-react";
import {
    AppState,
    DocPanelUICardInterface,
    PortalDocumentationStructureInterface
} from "../../../../features/core";
import {
    InboundProtocolListItemInterface,
    OIDCApplicationConfigurationInterface,
    SAMLApplicationConfigurationInterface,
    ApplicationManagementConstants,
    ApplicationManagementUtils
} from "../../../../features/applications";
import QuickStartIllustration from "../../../assets/images/illustrations/quick-start-illustration.svg";

/**
 * Proptypes for the applications help panel overview component.
 */
interface HelpPanelOverviewPropsInterface extends TestableComponentInterface {
    inboundProtocols?: InboundProtocolListItemInterface[];
    handleTabChange?: (tabId: number) => void;
    applicationType?: string;
    helpPanelDocStructure: PortalDocumentationStructureInterface;
    samples: DocPanelUICardInterface[];
    handleSelection: (sample) => void;
    onTOCClick?: (e: any, index: number) => void;
    handleIntegrateSelection?: (selection: string) => void;
}

/**
 * Applications Help Panel Overview Component.
 *
 * @param {HelpPanelOverviewPropsInterface} props - Props injected to the component.
 *
 * @return {React.ReactElement}
 */
export const HelpPanelOverview: FunctionComponent<HelpPanelOverviewPropsInterface> = (
    props: HelpPanelOverviewPropsInterface
): ReactElement => {

    const urlSearchParams = new URLSearchParams(location.search);

    const oidcConfigurations: OIDCApplicationConfigurationInterface = useSelector(
        (state: AppState) => state.application.oidcConfigurations);
    const samlConfigurations: SAMLApplicationConfigurationInterface = useSelector(
        (state: AppState) => state.application.samlConfigurations);
    const { t } = useTranslation();

    const { applicationType, inboundProtocols, samples, handleSelection, onTOCClick, handleIntegrateSelection } = props;

    const [ isOIDC, setIsOIDC ] = useState<boolean>(false);
    const [ isSAML, setIsSAML ] = useState<boolean>(false);
    const [ isOIDCConfigsLoading, setOIDCConfigsLoading ] = useState<boolean>(false);
    const [ isSAMLConfigsLoading, setSAMLConfigsLoading ] = useState<boolean>(false);
    const [ helpPanelSelectedSample, setHelpPanelSelectedSample ] = useState<DocPanelUICardInterface>(undefined);
    const [ selectedIntegration, setSelectedIntegration ] = useState<string>("INTEGRATE");

    useEffect(() => {
        if (inboundProtocols == undefined) {
            return;
        }

        if (inboundProtocols.length > 0) {
            inboundProtocols.map((protocol) => {
                if (protocol.type == "oauth2") {
                    setIsOIDC(true);
                } else if (protocol.type == "samlsso") {
                    setIsSAML(true);
                }
            })
        }

    }, [ inboundProtocols ]);

    useEffect(() => {
        if (oidcConfigurations !== undefined) {
            return;
        }

        ApplicationManagementUtils.getOIDCApplicationMeta()
            .finally(() => {
                setOIDCConfigsLoading(false);
            });
    }, [ oidcConfigurations ]);

    useEffect(() => {
        if (samlConfigurations !== undefined) {
            return;
        }

        ApplicationManagementUtils.getSAMLApplicationMeta()
            .finally(() => {
                setSAMLConfigsLoading(false);
            });
    }, [ samlConfigurations ]);

    return (
        <>
            <Grid>
            {
                    (urlSearchParams.get(ApplicationManagementConstants.APP_STATE_URL_SEARCH_PARAM_KEY)
                        === ApplicationManagementConstants.APP_STATE_URL_SEARCH_PARAM_VALUE) ?
                    <Grid.Row>
                        <Grid.Column width={ 12 }>
                            { /** TODO : Need to move inline styles to sperate file */}
                            <Heading as="h3" style={ { "margin" : "1em 0 0 0" } }>
                                Your Single Page Application has been registered!
                            </Heading>
                            <Header.Subheader
                                style={ { fontWeight: 200 } }>
                                Application has been configured for you with the recommended settings. Please select
                                    one of the following methods to get started.
                            </Header.Subheader>
                        </Grid.Column>
                    </Grid.Row> :
                    <Grid.Row>
                        <Grid.Column width={ 12 }>
                            { /** TODO : Need to move inline styles to sperate file */}
                            <Header.Subheader
                                style={ { fontWeight: 200 } }>
                                Please select one of the following methods to get started.
                            </Header.Subheader>
                        </Grid.Column>
                    </Grid.Row>
                }
                {
                    applicationType && ( applicationType == ApplicationManagementConstants.SPA 
                        || applicationType == "Web Application" )
                        ? (
                            <>
                                <Grid.Row>
                                    <Grid.Column width={ 8 } >
                                        <Card fluid>
                                            <Card.Content 
                                                onClick={ () => {
                                                    setSelectedIntegration("INTEGRATE");
                                                    handleIntegrateSelection("INTEGRATE");
                                                } } 
                                                className="selection-card-content"
                                            >
                                                <div className="integrate-radio">
                                                    <Radio checked={ selectedIntegration === "INTEGRATE" } />
                                                </div>
                                                <div className="card-content">
                                                    <Card.Header><span>Integrate</span> with your application</Card.Header>
                                                    <Card.Meta>
                                                        <span>Follow the below instruction to integrate your own application</span>
                                                    </Card.Meta>
                                                </div>
                                            </Card.Content>
                                        </Card>
                                    </Grid.Column>
                                    <Grid.Column width={ 8 } >
                                        <Card fluid>
                                            <Card.Content
                                                onClick={ () => {
                                                    setSelectedIntegration("TRY");
                                                    handleIntegrateSelection("TRY");
                                                } } 
                                                className="selection-card-content"
                                            >
                                                <div className="integrate-radio">
                                                    <Radio checked={ selectedIntegration === "TRY" } />
                                                </div>
                                                <div className="card-content">
                                                    <Card.Header><span>Try out </span>with a sample</Card.Header>
                                                    <Card.Meta>
                                                        <span>Use our pre configured samples and try out the authentication.</span>
                                                    </Card.Meta>
                                                </div>
                                            </Card.Content>
                                        </Card>
                                    </Grid.Column>
                                </Grid.Row>
                            </>
                        )
                        : null
                }
            </Grid>
        </>
    );
};
