/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import React, { FunctionComponent, ReactElement, useState, useEffect } from "react";
import { CodeEditor } from "@wso2is/react-components";
import { Divider, Tab } from "semantic-ui-react";
import { 
    ApplicationInterface,
    SupportedAuthProtocolTypes,
    getInboundProtocolConfig 
} from "../../../../features/applications";
import { useDispatch } from "react-redux";
import { useTranslation } from "react-i18next";
import { store } from "../../../../features/core";
import { addAlert } from "@wso2is/core/dist/src/store";
import { AlertLevels } from "@wso2is/core/dist/src/models";

interface TryOutComponentProps {
    application: ApplicationInterface;
}

/**
 * Sample onboarding flow component
 */
export const TryOutComponent: FunctionComponent<TryOutComponentProps> = (
    props: TryOutComponentProps
): ReactElement => {

    const {
        application
    } = props;

    const dispatch = useDispatch();
    const { t } = useTranslation();

    const deploymentConfigs = store.getState().config.deployment;
    const [ inboundProtocolConfigs, setInboundProtocolConfigs ] = useState(undefined);
    const [ authConfig, setAuthConfig ] = useState(undefined);
    const [ integrateCode, setIntegrateCode ] = useState("");

    // TODO : Move get configs to util function
    useEffect(() => {
        if ( application === undefined ) {
            return;
        }

        let protocolConfigs: any = {};
        const selectedProtocolList: string[] = [];

        application?.inboundProtocols?.map((protocol) => {

            const protocolName = mapProtocolTypeToName(protocol.type);

            getInboundProtocolConfig(application.id, protocolName)
                .then((response) => {
                    protocolConfigs = {
                        ...protocolConfigs,
                        [protocolName]: response
                    };

                    selectedProtocolList.push(protocolName);
                })
                .catch((error) => {
                    if (error?.response?.status === 404) {
                        return;
                    }

                    if (error?.response && error?.response?.data && error?.response?.data?.description) {
                        dispatch(addAlert({
                            description: error.response?.data?.description,
                            level: AlertLevels.ERROR,
                            message: t("devPortal:components.applications.notifications.getInboundProtocolConfig" +
                                ".error.message")
                        }));

                        return;
                    }

                    dispatch(addAlert({
                        description: t("devPortal:components.applications.notifications.getInboundProtocolConfig" +
                            ".genericError.description"),
                        level: AlertLevels.ERROR,
                        message: t("devPortal:components.applications.notifications.getInboundProtocolConfig" +
                            ".genericError.message")
                    }));
                })
                .finally(() => {
                    setInboundProtocolConfigs(protocolConfigs);
                });
        });
    },  [ application ]);

    useEffect(() => {
        if (inboundProtocolConfigs == undefined) {
            return;
        }

        const configs = {
            clientHost: deploymentConfigs.clientHost,
            clientID: inboundProtocolConfigs.oidc.clientId,
            loginCallbackURL: inboundProtocolConfigs.oidc.callbackURLs[0],
            logoutCallbackURL: inboundProtocolConfigs.oidc.callbackURLs[0],
            serverOrigin: deploymentConfigs.serverOrigin,
            tenant: deploymentConfigs.tenant,
            tenantPath: deploymentConfigs.tenantPath
        };

        setAuthConfig(configs);
        setIntegrateCode(`{
            loginCallbackURL: "` + configs?.loginCallbackURL + `",
            logoutCallbackURL: "` + configs?.logoutCallbackURL + `",
            clientHost: "` + configs?.clientHost + `",
            clientID: "` + configs?.clientID + `",
            serverOrigin: "` + configs?.serverOrigin + `"
        }
                            `);
    }, [ inboundProtocolConfigs ]);

    /**
     * Todo Remove this mapping and fix the backend.
     */
    const mapProtocolTypeToName = ((type: string): string => {
        let protocolName = type;
        if (protocolName === "oauth2") {
            protocolName = SupportedAuthProtocolTypes.OIDC;
        } else if (protocolName === "passivests") {
            protocolName = SupportedAuthProtocolTypes.WS_FEDERATION;
        } else if (protocolName === "wstrust") {
            protocolName = SupportedAuthProtocolTypes.WS_TRUST;
        } else if (protocolName === "samlsso") {
            protocolName = SupportedAuthProtocolTypes.SAML;
        }

        return protocolName;
    });


    // TODO : Sample code to be replaced with actual code in code editor
    const panes = [
        { menuItem: 'JSON', render: () => (
            <Tab.Pane>
                <CodeEditor
                    showLineNumbers={ false }
                    language="json"
                    sourceCode={ integrateCode }
                    options={ {
                        gutters: []
                    } }
                    theme={ "light" }
                    readOnly={ true }
                />
            </Tab.Pane>
        ) },
    ]

    return (
        <div className="quick-start-step-container">
            <div className="quick-start-step">
                <div className="step-content-container">
                    <h3>Download sample</h3>
                    <div className="step-content">
                        <p>Select the technology below to download</p>
                        
                    </div>
                    <Divider hidden/>
                </div>
            </div>
            <div className="quick-start-step">
                <div className="step-content-container">
                    <h3>Configure sample</h3>
                    <div className="step-content">
                        <p>Extract downloaded sample and copy and paste below config in 
                            <strong> src/config.json</strong>, or download and replace the file.</p>
                        <Divider hidden/>
                        <Tab menu={ { secondary: true, pointing: true } } panes={ panes } />
                    </div>
                    <Divider hidden/>
                </div>
            </div>
            <div className="quick-start-step">
                <div className="step-content-container">
                    <h3>Run</h3>
                    <div className="step-content">
                        <p>Run below command and visit: <a>http://localhost:5000</a></p>
                        <code>yarn start</code>
                    </div>
                    <Divider hidden/>
                </div>
            </div>
        </div>
    );
};
